import React from 'react';
import ReactDOM from 'react-dom';
import {
  select,
  geoPath,
  geoCentroid,
  geoNaturalEarth1,
  zoom,
  event,
  scaleOrdinal,
  scaleSequential,
  schemeSpectral,
  scaleSqrt,
  max,
  format
} from 'd3';

import { loadAndProcessData } from './loadAndProcessData';
import { sizeLegend } from './sizeLegend';
import { colorLegend } from './colorLegend';
import { colorHueLegend } from './colorHueLegend';

const svg = select('svg');

const projection = geoNaturalEarth1();
const pathGenerator = geoPath().projection(projection);
const radiusValue = d => d.properties['population'];
const tickText = d => d.properties['world_happiness_score'];
const score = d => d.properties['world_happiness_score_scaled'];
const country = d => d.properties['country'];

const g = svg.append('g');

g.append('path')
  .attr('class', 'sphere')
  .attr('d', pathGenerator({ type: 'Sphere' }));

svg.call(
  zoom().on('zoom', () => {
    g.attr('transform', event.transform);
  })
);

const populationFormat = format(".2s");
const colorLegFormat = format("");
const scoreFormat = format('.2f');

// sizeScale is for population
loadAndProcessData().then(countries => {
const sizeScale = scaleSqrt()
    .domain([0, max(countries.features, radiusValue)])
    .range([0, 25]);
  
var val = d3.interpolateInferno;
  
// scaleColor is for filling the country circle colors and legend
const scaleColor = d3.scaleSequential()
    .domain([0, 1])
    .interpolator(val); 
  
//  scoreScale is for the color legend ticks
const scoreScale = d3.scaleSequential()
    .domain([0, max(countries.features, tickText)])
	  .interpolator(val);
  
g.selectAll('path')
    .data(countries.features)
    .enter()
    .append('path')
    .attr('class', 'country')
    .attr('d', pathGenerator) 
    .attr('fill', d => (d.properties['population'] ? '#d9dbea' : '#d7e4f7'))
    .append('title')
    .text(d =>
      isNaN(radiusValue(d))
        ? 'Missing data'
        : [
            country(d),
            scoreFormat(tickText(d))
          ].join(': ')
    );

  countries.featuresWithPopulation.forEach(d => {
    d.properties.projected = projection(geoCentroid(d));
  });

  g.selectAll('circle')
    .data(countries.featuresWithPopulation)
    .enter()
    .append('circle')
    .attr('class', 'country-circle')
    .attr('cx', d => d.properties.projected[0])
    .attr('cy', d => d.properties.projected[1])
    .attr('r', d => sizeScale(radiusValue(d)))
    .style("fill", d => scaleColor(score(d))); 

  // Add size legend
  g.append('g')
    .attr(`transform`, `translate(43,80)`)
    .call(sizeLegend, {
      sizeScale,
      spacing: 26, 
      textOffset: 5,
      numTicks: 5,
      tickFormat: populationFormat
    })
    .append('text')
    .attr('class', 'legend-title')
    .text('Population')
    .attr('y', -30)
    .attr('x', -30);
  
//Add color legend
  g.append('g')
    .attr(`transform`, `translate(30,450)`)
    .call(colorLegend, {
      scaleColor,
      scoreScale,
      score,
      spacing: 24, 
      textOffset: 1,
      numTicks: 6,
      tickFormat: colorLegFormat
    })  
    .append('text')
    .attr('class', 'legend-title')
    .html('Happiness Score') 
    .attr('y', -30)
    .attr('x', -10)


  /*
       groups.selectAll("circle")
   // .attr("fill", d => colorScale(d).interpolator);
      .data()
      .style("fill", d => scaleColor(score(d))); 
  
  g.select('#wrapper')
	.selectAll('circle')
	.data(myData)
	.enter()
	.append('circle')
	.attr('r', 10)
	.attr('cx', function(d) {
		return linearScale(d);
	})
	.style('fill', function(d) {
		return sequentialScale(d);
	});
  */
    
  
});
