import React from 'react';
import ReactDOM from 'react-dom';
import {
  select,
  geoPath,
  geoCentroid,
  geoNaturalEarth1,
  zoom,
  event,
  scaleOrdinal,
  scaleSequential,
  schemeSpectral,
  scaleSqrt,
  max,
  format
} from 'd3';

import { loadAndProcessData } from './loadAndProcessData';
import { sizeLegend } from './sizeLegend';
import { colorLegend } from './colorLegend';

const svg = select('svg');

const margin = { top: 10, right: 30, bottom: 65, left: 90 };
const innerHeight = 500 - margin.top - margin.bottom;
const innerWidth = 960 - margin.left - margin.right;
const legendOffsetX = innerWidth * 2 + 50;
const legendOffsetY = innerHeight;

const projection = geoNaturalEarth1();
const pathGenerator = geoPath().projection(projection);
const radiusValue = d => d.properties['population'];
const tickText = d => d.properties['world_happiness_score'];
const score = d => d.properties['world_happiness_score_scaled'];
const country = d => d.properties['country'];

const g = svg.append('g');

var data = [{"color":"#000004","value":0},
                {"color":"#140b34","value":25},
                {"color":"#3d0965","value":50},
                {"color":"#65156e","value":75},
                {"color":"#8d2369","value":100},
                {"color":"#a32c61","value":110},
                {"color":"#b43359","value":125},
                {"color":"#d74b3f","value":140},
                {"color":"#e15635","value":140},
                {"color":"#ef6e21","value":165},
                {"color":"#fb9d07","value":200},
                {"color":"#f8cd37","value":205},
                {"color":"#f6d746","value":205},
                {"color":"#f6d543","value":215},
                {"color":"#f5db4c","value":225},
                {"color":"#f3e55d","value":226},
                {"color":"#f1ed71","value":226},
                {"color":"#f1f179","value":230},
                {"color":"#fcffa4","value":250}];
   
    var extent = d3.extent(data, d => d.value);
    var defs = svg.append("defs");
    var linearGradient = defs.append("linearGradient").attr("id", "myGradient");
    linearGradient.selectAll("stop")
        .data(data)
        .enter().append("stop")
        .attr("offset", d => ((d.value - extent[0]) / (extent[1] - extent[0]) * 100) + "%")
        .attr("stop-color", d => d.color);

g.append('path')
  .attr('class', 'sphere')
  .attr('d', pathGenerator({ type: 'Sphere' }));

svg.call(
  zoom().on('zoom', () => {
    g.attr('transform', event.transform);
  })
);

const populationFormat = format(".2s");
const colorLegFormat = format("");
const scoreFormat = format('.2f');

// sizeScale is for population
loadAndProcessData().then(countries => {
const sizeScale = scaleSqrt()
    .domain([0, max(countries.features, radiusValue)])
    .range([0, 25]);
  
var val = d3.interpolateInferno;
  
// scaleColor is for filling the country circle colors and legend
const scaleColor = d3.scaleSequential()
    .domain([0, 1])
    .interpolator(val); 
  
//  scoreScale is for the color legend ticks
const scoreScale = d3.scaleLinear()
    .domain([0, max(countries.features, tickText)])
    .range(0, 200);
  
g.selectAll('path')
    .data(countries.features)
    .enter()
    .append('path')
    .attr('class', 'country')
    .attr('d', pathGenerator) 
    .attr('fill', d => (d.properties['population'] ? '#d9dbea' : '#d7e4f7'))
    .append('title')
    .text(d =>
      isNaN(radiusValue(d))
        ? 'Missing data'
        : [
            country(d),
            scoreFormat(tickText(d))
          ].join(': ')
    );

  countries.featuresWithPopulation.forEach(d => {
    d.properties.projected = projection(geoCentroid(d));
  });

  g.selectAll('circle')
    .data(countries.featuresWithPopulation)
    .enter()
    .append('circle')
    .attr('class', 'country-circle')
    .attr('cx', d => d.properties.projected[0])
    .attr('cy', d => d.properties.projected[1])
    .attr('r', d => sizeScale(radiusValue(d)))
    .style("fill", d => scaleColor(score(d))); 

  // Add size legend
  g.append('g')
    .attr(`transform`, `translate(43,80)`)
    .call(sizeLegend, {
      sizeScale,
      spacing: 26, 
      textOffset: 5,
      numTicks: 5,
      tickFormat: populationFormat
    })
    .append('text')
    .attr('class', 'legend-title')
    .text('Population')
    .attr('y', -30)
    .attr('x', -30);
  
   g.append('rect')
        .attr("x",140)
        .attr("y",440)
        .attr("width",215)
        .attr("height",25)
        .style("fill", "url(#myGradient)");
  
   g.append('g')
    .attr(`transform`, `translate(25,450)`)
    .append('text')
    .attr('class', 'legend-title')
    .html('Less Happy') 
    .attr('y', 10)
    .attr('x', -15);
  g.append('g')
    .attr(`transform`, `translate(25,450)`)
    .append('text')
    .attr('class', 'legend-title')
    .html('More Happy') 
    .attr('y', 10)
    .attr('x', 330);
  
});
